///////////////////////////////////////////////////////////////////////
//
//	shlext.h
//
//	Definitions of Undocumented Shell Structures
//
//  The latest copy of this file should be available at:
//  <http://users.lia.net/chris/shellext.zip>
//
//  History:
//    20/11/97  Removed IID_IShellUndoc (was actually IID_IDropTarget!)
//    20/11/97  Chopped further irrelevant CABVIEW related items
//    19/11/97	Added some dox for SHShellFolderView_Message
//    15/11/97  Fixed IShellDetails interface
//     7/10/97	removed stuff already availible in <shlobj.h> and <shlguid.h>
//     7/10/97	documented some ILxxx Functions, added annotations. changed
//              SHCreateShellViewEx structure with new info...
//    25/ 2/97  added default shell menu id's
//    22/ 2/97  removed cabobj - suppose this makes us incompatable with vc prior to 4.2
//    24/ 1/97  added annotations. minor mod's
//     9/ 1/97  changed SHCreateShellViewEx structure with new info...
//    13/12/96  added _all_ the undocumented shell32 exports
//    12/12/96  removed the IShellViewCallback Class
//    ??/??/??  added the stuff hacked from the CABView example MS released
//
//  If you have any changes to make to this file, feel free to mail them
//  to me at <mailto:chris@dbn.lia.net>, so I can integrate them with the
//  original available on my homepage.
//
///////////////////////////////////////////////////////////////////////
#ifndef _SHLEXT_H_
#define _SHLEXT_H_

#include <shlobj.h>

#ifdef __cplusplus
extern "C" {
#endif

// Here i'm doing a bad thing, but its a damn pain trying to get the
// include order sorted out otherwise
#define DEFINE_SHLGUID(name, l, w1, w2) DEFINE_GUID(name, l, w1, w2, 0xC0,0,0,0,0,0,0,0x46)

//
// Interface IDs    xx=E0-FF
//
// Missing IID's: E0, E7, F4-FF
DEFINE_SHLGUID(IID_IShellDetails,         0x000214ECL, 0, 0);
DEFINE_SHLGUID(IID_IDelayedRelease,       0x000214EDL, 0, 0);

//The Briefcase is a shell extension...
DEFINE_GUID(IID_IBriefcaseStg,            0x8BCE1FA1L, 0x0921, 0x101B, 0xB1, 0xFF, 0x00, 0xDD, 0x01, 0x0C, 0xCC, 0x48);
DEFINE_GUID(IID_IShellDirectory,          0xF3364BA0L, 0x65B9, 0x11CE, 0xA9, 0xBA, 0x00, 0xAA, 0x00, 0x4A, 0xE8, 0x37);

///////////////////////////////////////////////////////////////////////
// typeDefine the undocumented interfaces
//
typedef interface IShellDetails      IShellDetails;


//==========================================================================
// Stuff used by the IShellBrowser/IShellView objects.
//==========================================================================

// Defines the range of command IDs used by view windows.
//		FCIDM_SHVIEWFIRST/LAST	for the right pane (IShellView)
#define FCIDM_SHVIEWFIRST           0x0000
#define FCIDM_SHVIEWLAST            0x7fff
//		FCIDM_BROWSERFIRST/LAST	for the explorer frame (IShellBrowser)
#define FCIDM_BROWSERFIRST          0xa000
#define FCIDM_BROWSERLAST           0xbf00
//		FCIDM_GLOBAL/LAST		for the explorer's submenu IDs
#define FCIDM_GLOBALFIRST           0x8000
#define FCIDM_GLOBALLAST            0x9fff

// Global submenu IDs
#define FCIDM_MENU_FILE		(FCIDM_GLOBALFIRST+0x0000)
#define FCIDM_MENU_EDIT		(FCIDM_GLOBALFIRST+0x0040)
#define FCIDM_MENU_VIEW		(FCIDM_GLOBALFIRST+0x0080)
#define FCIDM_MENU_TOOLS	(FCIDM_GLOBALFIRST+0x00c0)
#define FCIDM_MENU_HELP		(FCIDM_GLOBALFIRST+0x0100)

// The shell handles internally all the following ID's without calling
// InvokeCommand()
#define FCIDM_SHVIEWSHELL           0x7000

// File menu options
#define FCIDM_MENU_FILE_CREATESHORTCUT   (FCIDM_SHVIEWSHELL + 0x0010) //28688
#define FCIDM_MENU_FILE_DELETE           (FCIDM_SHVIEWSHELL + 0x0011) //28689
#define FCIDM_MENU_FILE_RENAME           (FCIDM_SHVIEWSHELL + 0x0012) //28690
#define FCIDM_MENU_FILE_PROPERTIES       (FCIDM_SHVIEWSHELL + 0x0013) //28691

// Edit menu options
#define FCIDM_MENU_EDIT_CUT              (FCIDM_SHVIEWSHELL + 0x0018) //28696
#define FCIDM_MENU_EDIT_COPY             (FCIDM_SHVIEWSHELL + 0x0019) //28697
#define FCIDM_MENU_EDIT_PASTE            (FCIDM_SHVIEWSHELL + 0x001A) //28698
#define FCIDM_MENU_EDIT_UNDO             (FCIDM_SHVIEWSHELL + 0x001B) //28699
#define FCIDM_MENU_EDIT_PASTESHORTCUT    (FCIDM_SHVIEWSHELL + 0x001C) //28700

// View Report styles
#define FCIDM_MENU_VIEW_SELECTALL        (FCIDM_SHVIEWSHELL + 0x0021) //28705
#define FCIDM_MENU_VIEW_INVERTSELECTION  (FCIDM_SHVIEWSHELL + 0x0022) //28706

#define FCIDM_MENU_VIEW_LARGEICONS       (FCIDM_SHVIEWSHELL + 0x0029) //28713
#define FCIDM_MENU_VIEW_SMALLICONS       (FCIDM_SHVIEWSHELL + 0x002A) //28714
#define FCIDM_MENU_VIEW_LIST             (FCIDM_SHVIEWSHELL + 0x002B) //28715
#define FCIDM_MENU_VIEW_DETAILS          (FCIDM_SHVIEWSHELL + 0x002C) //28716

#define FCIDM_MENU_VIEW_AUTOARRANGE      (FCIDM_SHVIEWSHELL + 0x0031) //28721
#define FCIDM_MENU_VIEW_LINEUPICONS      (FCIDM_SHVIEWSHELL + 0x0032) //28722

// Help menu
#define FCIDM_MENU_HELP_HELPTOPICS       (FCIDM_SHVIEWSHELL + 0x0041) //28737

typedef LPBYTE LPVIEWSETTINGS;

// Values for uWindow parameter of GetWindow/GetWindowRect member function.
#define FCW_STATUS              0x0001
#define FCW_TOOLBAR             0x0002

// SetToolbarItems flags
#define FCT_MERGE               0x0001
#define FCT_CONFIGABLE		0x0002
#define FCT_ADDTOEND		0x0004

// shellview select item flags
#define SVSI_DESELECT		0x0000
#define SVSI_SELECT             0x0001
#define SVSI_EDIT               0x0003  // includes select
#define SVSI_DESELECTOTHERS     0x0004
#define SVSI_ENSUREVISIBLE      0x0008

#define SFVID_FIRST (FCIDM_SHVIEWFIRST + 0x2000)
#define SFVID_LAST  (FCIDM_SHVIEWFIRST + 0x3000)

#define SFVID_MENU_ARRANGE (SFVID_FIRST + 0x000)


//Column Info for IShellDetails & ShellView CallBack
typedef struct
{
	DWORD   fJustify;
	INT	nWidth;
	STRRET	text;
} SHColInfo, *PSHCOLINFO;

/////////////////////////////////////////////////////////////////////////////
// ShellFolderViewCallback

typedef HRESULT(CALLBACK *SHELLVIEWPROC)(
  DWORD dwUserParam,
  LPSHELLFOLDER psf,
  HWND hwnd,
  UINT uMsg,
  WPARAM wParam,
  LPARAM lParam);

// NF valid values for the "viewmode" item of the SHELLTEMPLATE
#define NF_INHERITVIEW	  0x0000
#define NF_LOCALVIEW	    0x0001


typedef struct _SHELLVIEWDATA	// idl
{
	DWORD           dwSize;
	LPSHELLFOLDER   pShellFolder;
	DWORD           dwUserParam;
	LPCITEMIDLIST   pidl;
	DWORD           v3;        // always 0
	SHELLVIEWPROC   pCallBack;
	DWORD           viewmode;  // NF_* enum
} SHELLVIEWDATA, * LPSHELLVIEWDATA;
typedef const SHELLVIEWDATA * LPCSHELLVIEWDATA;


typedef struct tag_SFVCB_COLUMNINFOSTRUCT
{
  LPCITEMIDLIST pidl;  // NULL if column header requested, else
                       // a simple pidl to the item whose details are needed.
  SHColInfo	sci;   // return filled in with the details.
} SFVCB_COLUMNINFOSTRUCT,* LPSFVCB_COLUMNINFOSTRUCT, const SFVCB_COLUMNINFOSTRUCT* LPCSFVCB_COLUMNINFOSTRUCT;

//Msg   wParam    lParam    Called from / Notes
//0x01  0         00d3f45c  CDefView_OnActivate
//0x05  0         struct?   DefView_MergeToolBar
//0x06  nID       HMENU?    DefView_MergeToolbar
//0x07  nID       HMENU     OnInitMenuPopup
//0x08  item,nID  HMENU?    selected item changed.
//0x0b  0         0         DefView_Command
//0x0c  0         0         CDefView_Release
//0x0f  hwndView  struct?   CDefView_CreateViewWindow
//0x10  hwndView  0         CDefView_DestroyViewWindow
//0x11  0         0         DefView_FillDone
//0x12  0         0         CDefView_WndProc
//0x14  0         0         CDefView_CreateViewWindow
//0x17  column    struct    GetDetailsOf
//0x18  0         column    ColumnClick
//0x19  0         struct?   CDefView_CreateViewWindow
//0x1d  pidl?     struct?   DefView_AddObject
//0x1e  0         struct?   DefView_RemoveObject
//                          CDefView_OnDeactivate
//0x1f  1         NULL      DV_UpdateStatusBar
//0x20  0         0         DefView_FillObjects
//0x22  0         struct?   DefView_AddColumns
//      1         struct?   DefView_SaveCols
//0x25  0         0         DSV_FilterDPAs

#define SFVCB_GETDETAILSOF  0x17
// wParam is the index of the column
// lParam is LPSFVCB_COLUMNINFOSTRUCT
// return NOERROR if the column is valid. Else return OLE error.
#define SFVCB_COLUMNCLICK   0x18
// wParam is NULL
// lParam is the column index.
// return NOERROR
#define SFVCB_VIEWRELEASE   0x0c
// SFVM_VIEWRELEASE message indicates that the IShellView object is being
// released.
#define SFVCB_ONCREATE      0x0f
// Called to indicate the view window has been created.
// wParam is the views HWND
// lParam is ???
 

WINSHELLAPI HRESULT WINAPI SHCreateShellFolderViewEx(
  LPSHELLVIEWDATA psvcbi, //[in ] shelltemplate struct
  LPVOID* ppv);           //[out] IShellView pointer

/////////////////////////////////////////////////////////////////////////////
// IShellDetails

#undef  INTERFACE
#define INTERFACE   IShellDetails

DECLARE_INTERFACE_(IShellDetails, IUnknown)
{
	// IUnknown methods
	STDMETHOD(QueryInterface)(THIS_ REFIID riid, LPVOID FAR* ppvObj) PURE;
	STDMETHOD_(ULONG,AddRef)(THIS)  PURE;
	STDMETHOD_(ULONG,Release)(THIS) PURE;

	// IShellDetails methods
	STDMETHOD(GetDetailsOf)(THIS_ LPCITEMIDLIST pidl, UINT col, SHColInfo *data) PURE;
	STDMETHOD(ColumnClick)(THIS_ UINT col) PURE;
};

typedef IShellDetails FAR* LPSHELLDETAILS;

/////////////////////////////////////////////////////////////////////////////
// Totally miscellaneous stuff

// ??? about IShellFolder ?
/* This is the interface for a browser to "subclass" the main File Cabinet
** window.  Note that only the hwnd, message, wParam, and lParam fields of
** the msg structure are used.  The browser window will get a WM_NOTIFY
** message with NULL ID, FCN_MESSAGE as the code, and a far pointer to
** FCMSG_NOTIFY as the lParam.
*/
typedef struct tagFCMSG_NOTIFY
{
        NMHDR   hdr;
        MSG     msg;
        LRESULT lResult;
} FCMSG_NOTIFY;

#define FCN_MESSAGE     (100)


///////////////////////////////////////////////////////////////////////////////
//
// Homeless things.
//
///////////////////////////////////////////////////////////////////////////////

//---------------------------------------------------------------------------
// messages that can be send to the cabinet by other apps
//
//---------------------------------------------------------------------------

// Change the path of an existing folder.
// wParam:
//	0:		LPARAM is a string, handle the message immediately.
//	CSP_HANDLE:	LPARAM is a handle. handle the message immediatelt
//			and then free the handle.
//	CSP_REPOST:	LPARAM is a string, copy the string and handle the
// 			message later.
// 	CSP_REPOST|CSP_HANDLE:
//			LPARAM is a handle, just handle the message later
//			and free the handle then.
// lParam: LPSTR or HANDLE of path.
//
#define CSP_REPOST				0x0001
#define CSP_HANDLE				0x0002

#define CWM_SETPATH				(WM_USER + 2)

// lpsv points to the Shell View extension that requested idle processing
// uID is an app define identifier for the processor
// returns: TRUE if there is more idle processing necessary, FALSE if all done
// Note that the idle processor should do one "atomic" operation and return
// as soon as possible.
typedef BOOL (CALLBACK *FCIDLEPROC)(void FAR *lpsv, UINT uID);

// Inform the File Cabinet that you want idle messages.
// This should ONLY be used by File Cabinet extensions.
// wParam: app define UINT (passed to FCIDLEPROC).
// lParam: pointer to an FCIDLEPROC.
// return: TRUE if successful; FALSE otherwise
#define CWM_WANTIDLE            (WM_USER + 3)
#define FileCabinet_WantIdle(_hwnd, _user, _lpidlproc) SendMessage(_hwnd, CWM_WANTIDLE, (WPARAM)_user, (LPARAM)_lpidlproc)

// get or set the FOLDERSETTINGS for a view
// wParam: BOOL TRUE -> set to view info buffer, FALSE -> get view info buffer
// lParam: LPFOLDERSETTINGS buffer to get or set view info
#define CWM_GETSETCURRENTINFO	(WM_USER + 4)
#define FileCabinet_GetSetCurrentInfo(_hwnd, _bSet, _lpfs) SendMessage(_hwnd, CWM_GETSETCURRENTINFO, (WPARAM)(_bSet), (LPARAM)(LPFOLDERSETTINGS)_lpfs)

// selects the specified item in the current view
// wParam: BOOL TRUE -> select, FALSE -> deselect
// lParam: LPCSTR of the item ID (not display name), NULL -> all items
#define CWM_SELECTITEM          (WM_USER + 5)
#define FileCabinet_SelectItem(_hwnd, _bSel, _lpidl) SendMessage(_hwnd, CWM_SELECTITEM, (WPARAM)_bSel, (LPARAM)_lpidl)

// tells the window to punt its wait cursor.  used for handoff
// while thread inits
#define CWM_STOPWAITING         (WM_USER + 6)
#define FileCabinet_StopWaiting(_hwnd) SendMessage(_hwnd, CWM_STOPWAITING, 0, 0L)

// Get the IShellBrowser object associated with an hwndMain
#define CWM_GETISHELLBROWSER	(WM_USER + 7)
#define FileCabinet_GetIShellBrowser(_hwnd) (IShellBrowser FAR *)SendMessage(_hwnd, CWM_GETISHELLBROWSER, 0, 0L)

//---------------------------------------------------------------------------
// All the "undocumented" shell32 exports.
//
//---------------------------------------------------------------------------

// The Shell's Default Task allocator
WINSHELLAPI void WINAPI SHFree( LPVOID);
WINSHELLAPI LPVOID WINAPI SHAlloc( UINT);

///////////////////////////////////////////////////////////////////////////////
// Cool helper functions to work with IDLists

// free an idlist using the shell allocater
// (how is this func diffrent from SHFree?
WINSHELLAPI void WINAPI ILFree(LPITEMIDLIST pidl);

// dupicate an idlist
WINSHELLAPI LPITEMIDLIST WINAPI ILClone(LPCITEMIDLIST pidl);

// Adds the single item to the idlist indicated by pidl.
// if bEnd is 0, adds the item to the front of the list,
// otherwise adds the item to the end.
// NOTE: Destroys the passed in idlist!
WINSHELLAPI LPITEMIDLIST WINAPI ILAppendID(LPITEMIDLIST pidl,LPCITEMIDLIST item,BOOL bEnd);

// duplicates the first idlist of a complex pidl
WINSHELLAPI LPITEMIDLIST WINAPI ILCloneFirst(LPCITEMIDLIST pidl);

// Concatenates two complex idlists.
// The pidl is the first one, pidlsub the next one
// NOTE: Does not destroy the passed in idlists!
WINSHELLAPI LPITEMIDLIST WINAPI ILCombine(LPCITEMIDLIST pidl, LPCITEMIDLIST pidlsub);

WINSHELLAPI int WINAPI ILCreateFromPath(int);
WINSHELLAPI int WINAPI ILFindChild(int,int);
WINSHELLAPI int WINAPI ILFindLastID(int);
WINSHELLAPI int WINAPI ILGetDisplayName(int,int);

// returns the next pidl in a complex idlist
WINSHELLAPI LPITEMIDLIST WINAPI ILGetNext(LPCITEMIDLIST);

// Gets the byte size of an idlist including zero terminator.
WINSHELLAPI UINT WINAPI ILGetSize(LPCITEMIDLIST pidl);

// Used to implement explorer parameters?
WINSHELLAPI int WINAPI ILGlobalClone(int);
WINSHELLAPI int WINAPI ILGlobalFree(int);


WINSHELLAPI int WINAPI ILIsEqual(int,int);
WINSHELLAPI int WINAPI ILIsParent(int,int,int);
WINSHELLAPI int WINAPI ILLoadFromStream(int,int);

// Creates a new list with the last tiem removed.
WINSHELLAPI LPITEMIDLIST WINAPI ILRemoveLastID(LPCITEMIDLIST);

WINSHELLAPI int WINAPI ILSaveToStream(int,int);

///////////////////////////////////////////////////////////////////////////////
WINSHELLAPI int WINAPI AddCommasW(int,int);
WINSHELLAPI int WINAPI ArrangeWindows(int,int,int,int,int);
WINSHELLAPI int WINAPI CIDLData_CreateFromIDArray(int,int,int,int);
WINSHELLAPI int WINAPI CallCPLEntry16(int,int,int,int,int,int);
WINSHELLAPI int WINAPI CheckEscapesA(int,int);
WINSHELLAPI int WINAPI CheckEscapesW(int,int);
//CommandLineToArgvW(int,int);
WINSHELLAPI int WINAPI Control_FillCache_RunDLL(int,int,int,int);
WINSHELLAPI int WINAPI Control_FillCache_RunDLLA(int,int,int,int);
WINSHELLAPI int WINAPI Control_FillCache_RunDLLW(int,int,int,int);
WINSHELLAPI int WINAPI Control_RunDLL(int,int,int,int);
WINSHELLAPI int WINAPI Control_RunDLLA(int,int,int,int);
WINSHELLAPI int WINAPI Control_RunDLLW(int,int,int,int);
WINSHELLAPI int WINAPI DAD_AutoScroll(int,int,int);
WINSHELLAPI int WINAPI DAD_DragEnter(int);
WINSHELLAPI int WINAPI DAD_DragEnterEx(int,int,int);
WINSHELLAPI int WINAPI DAD_DragLeave(void);
WINSHELLAPI int WINAPI DAD_DragMove(int,int);
WINSHELLAPI int WINAPI DAD_SetDragImage(int,int);
WINSHELLAPI int WINAPI DAD_SetDragImageFromListView(int,int,int);
WINSHELLAPI int WINAPI DAD_ShowDragImage(int);
WINSHELLAPI int WINAPI Desktop_UpdateBriefcaseOnEvent(int,int);
WINSHELLAPI int WINAPI DoEnvironmentSubstA(int,int);
WINSHELLAPI int WINAPI DoEnvironmentSubstW(int,int);
//DragAcceptFiles(int,int);
//DragFinish(int);
//DragQueryFileA(int,int,int,int);
WINSHELLAPI int WINAPI DragQueryFileAorW(int,int,int,int,int,int);
//DragQueryFileW(int,int,int,int);
WINSHELLAPI int WINAPI DragQueryInfo(int,int);
//DragQueryPoint(int,int);
WINSHELLAPI int WINAPI DriveType(int);
WINSHELLAPI int WINAPI DuplicateIcon(int,int);
WINSHELLAPI int WINAPI ExitWindowsDialog(int);
//ExtractAssociatedIconA(int,int,int);
WINSHELLAPI int WINAPI ExtractAssociatedIconExA(int,int,int,int);
WINSHELLAPI int WINAPI ExtractAssociatedIconExW(int,int,int,int);
//ExtractAssociatedIconW(int,int,int);
//ExtractIconA(int,int,int);
//ExtractIconEx(int,int,int,int,int);
//ExtractIconExA(int,int,int,int,int);
//ExtractIconExW(int,int,int,int,int);
WINSHELLAPI int WINAPI ExtractIconResInfoA(int,int,int,int,int);
WINSHELLAPI int WINAPI ExtractIconResInfoW(int,int,int,int,int);
//ExtractIconW(int,int,int);
WINSHELLAPI int WINAPI ExtractVersionResource16W(int,int);
WINSHELLAPI int WINAPI FOOBAR1217(int,int,int);
WINSHELLAPI int WINAPI FileIconInit(int);
WINSHELLAPI int WINAPI FileMenu_AbortInitMenu(void);
WINSHELLAPI int WINAPI FileMenu_AddFilesForPidl(int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI FileMenu_AppendFilesForPidl(int,int,int);
WINSHELLAPI int WINAPI FileMenu_AppendItem(int,int,int,int,int,int);
WINSHELLAPI int WINAPI FileMenu_Create(int,int,int,int,int);
WINSHELLAPI int WINAPI FileMenu_DeleteAllItems(int);
WINSHELLAPI int WINAPI FileMenu_DeleteItemByCmd(int,int);
WINSHELLAPI int WINAPI FileMenu_DeleteItemByIndex(int,int);
WINSHELLAPI int WINAPI FileMenu_DeleteMenuItemByFirstID(int,int);
WINSHELLAPI int WINAPI FileMenu_DeleteSeparator(int);
WINSHELLAPI int WINAPI FileMenu_Destroy(int);
WINSHELLAPI int WINAPI FileMenu_DrawItem(int,int);
WINSHELLAPI int WINAPI FileMenu_EnableItemByCmd(int,int,int);
WINSHELLAPI int WINAPI FileMenu_FindSubMenuByPidl(int,int);
WINSHELLAPI int WINAPI FileMenu_GetItemExtent(int,int);
WINSHELLAPI int WINAPI FileMenu_GetLastSelectedItemPidls(int,int,int);
WINSHELLAPI int WINAPI FileMenu_HandleMenuChar(int,int);
WINSHELLAPI int WINAPI FileMenu_InitMenuPopup(int);
WINSHELLAPI int WINAPI FileMenu_InsertUsingPidl(int,int,int,int,int,int);
WINSHELLAPI int WINAPI FileMenu_Invalidate(int);
WINSHELLAPI int WINAPI FileMenu_MeasureItem(int,int);
WINSHELLAPI int WINAPI FileMenu_ReplaceUsingPidl(int,int,int,int,int);
WINSHELLAPI int WINAPI FileMenu_TrackPopupMenuEx(int,int,int,int,int,int);
WINSHELLAPI int WINAPI FindExeDlgProc(int,int,int,int);
//FindExecutableA(int,int,int);
//FindExecutableW(int,int,int);
WINSHELLAPI int WINAPI FreeIconList(int,int);
WINSHELLAPI int WINAPI GetFileNameFromBrowse(int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI Int64ToString(int,int,int,int,int,int);
WINSHELLAPI int WINAPI InternalExtractIconListA(int,int,int);
WINSHELLAPI int WINAPI InternalExtractIconListW(int,int,int);
WINSHELLAPI int WINAPI InvalidateDriveType(int);
WINSHELLAPI int WINAPI IsLFNDrive(int);
WINSHELLAPI int WINAPI IsNetDrive(int);
WINSHELLAPI int WINAPI IsUserAnAdmin(void);
WINSHELLAPI int WINAPI LargeIntegerToString(int,int,int,int,int);
WINSHELLAPI int WINAPI Link_AddExtraDataSection(int,int);
WINSHELLAPI int WINAPI Link_ReadExtraDataSection(int,int);
WINSHELLAPI int WINAPI Link_RemoveExtraDataSection(int,int);
WINSHELLAPI int WINAPI NTSHChangeNotifyDeregister(int);
WINSHELLAPI int WINAPI NTSHChangeNotifyRegister(int,int,int,int,int,int);
WINSHELLAPI int WINAPI OleStrToStrN(int,int,int,int);
WINSHELLAPI int WINAPI OpenAs_RunDLL(int,int,int,int);
WINSHELLAPI int WINAPI OpenAs_RunDLLA(int,int,int,int);
WINSHELLAPI int WINAPI OpenAs_RunDLLW(int,int,int,int);
WINSHELLAPI int WINAPI OpenRegStream(int,int,int,int);
WINSHELLAPI int WINAPI ParseField(int,int,int,int);
WINSHELLAPI int WINAPI PathAddBackslash(int);
WINSHELLAPI int WINAPI PathAppend(int,int);
WINSHELLAPI int WINAPI PathBuildRoot(int,int);
WINSHELLAPI int WINAPI PathCleanupSpec(int,int);
WINSHELLAPI int WINAPI PathCombine(int,int,int);
WINSHELLAPI int WINAPI PathFileExists(int);
WINSHELLAPI int WINAPI PathFindExtension(int);
WINSHELLAPI int WINAPI PathFindFileName(int);
WINSHELLAPI int WINAPI PathFindOnPath(int,int);
WINSHELLAPI int WINAPI PathGetArgs(int);
WINSHELLAPI int WINAPI PathGetDriveNumber(int);
WINSHELLAPI int WINAPI PathGetExtension(int,int,int);
WINSHELLAPI int WINAPI PathGetShortPath(int);
WINSHELLAPI int WINAPI PathIsDirectory(int);
WINSHELLAPI int WINAPI PathIsExe(int);
WINSHELLAPI int WINAPI PathIsRelative(int);
WINSHELLAPI int WINAPI PathIsRoot(int);
WINSHELLAPI int WINAPI PathIsSameRoot(int,int);
WINSHELLAPI int WINAPI PathIsUNC(int);
WINSHELLAPI int WINAPI PathMakeUniqueName(int,int,int,int,int);
WINSHELLAPI int WINAPI PathMatchSpec(int,int);
WINSHELLAPI int WINAPI PathQualify(int);
WINSHELLAPI int WINAPI PathQuoteSpaces(int);
WINSHELLAPI int WINAPI PathRemoveBlanks(int);
WINSHELLAPI int WINAPI PathRemoveFileSpec(int);
WINSHELLAPI int WINAPI PathResolve(int,int,int);
WINSHELLAPI int WINAPI PathSetDlgItemPath(int,int,int);
WINSHELLAPI int WINAPI PathStripPath(int);
WINSHELLAPI int WINAPI PathStripToRoot(int);
WINSHELLAPI int WINAPI PathUnquoteSpaces(int);
WINSHELLAPI int WINAPI PathYetAnotherMakeUniqueName(int,int,int,int);
WINSHELLAPI int WINAPI PickIconDlg(int,int,int,int);
WINSHELLAPI int WINAPI PifMgr_CloseProperties(int,int);
WINSHELLAPI int WINAPI PifMgr_GetProperties(int,int,int,int,int);
WINSHELLAPI int WINAPI PifMgr_OpenProperties(int,int,int,int);
WINSHELLAPI int WINAPI PifMgr_SetProperties(int,int,int,int,int);
WINSHELLAPI int WINAPI Printer_AddPrinterPropPages(int,int);
WINSHELLAPI int WINAPI Printer_LoadIconsW(int,int,int);
WINSHELLAPI int WINAPI PrintersGetCommand_RunDLL(int,int,int,int);
WINSHELLAPI int WINAPI PrintersGetCommand_RunDLLA(int,int,int,int);
WINSHELLAPI int WINAPI PrintersGetCommand_RunDLLW(int,int,int,int);
WINSHELLAPI int WINAPI Printers_GetPidl(int,int);
WINSHELLAPI int WINAPI Printers_RegisterWindowW(int,int,int,int);
WINSHELLAPI int WINAPI Printers_UnregisterWindow(int);
WINSHELLAPI int WINAPI RLBuildListOfPaths(void);
WINSHELLAPI int WINAPI ReadCabinetState(int);
WINSHELLAPI int WINAPI RealDriveType(int,int);
WINSHELLAPI int WINAPI RealDriveTypeFlags(int,int);
WINSHELLAPI int WINAPI RealShellExecuteA(int,int,int,int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI RealShellExecuteExA(int,int,int,int,int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI RealShellExecuteExW(int,int,int,int,int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI RealShellExecuteW(int,int,int,int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI ReceiveAddToRecentDocs(int,int);
WINSHELLAPI int WINAPI RegenerateUserEnvironment(int,int);
WINSHELLAPI int WINAPI RegisterShellHook(int,int);
WINSHELLAPI int WINAPI RestartDialog(int,int,int);
WINSHELLAPI int WINAPI RunDll_CallEntry16(int,int,int,int,int);
WINSHELLAPI int WINAPI RunFileDlg(int,int,int,int,int,int);
WINSHELLAPI int WINAPI SHAbortInvokeCommand(void);
WINSHELLAPI int WINAPI SHAddFromPropSheetExtArray(int,int,int);
//SHAddToRecentDocs(int,int);
//WINSHELLAPI int WINAPI SHAlloc(int);
WINSHELLAPI int WINAPI SHAllocShared(int,int,int);
//SHAppBarMessage(int,int);
//SHBrowseForFolder(int);
//SHBrowseForFolderA(int);
//SHBrowseForFolderW(int);
WINSHELLAPI int WINAPI SHCLSIDFromString(int,int);
WINSHELLAPI int WINAPI SHChangeNotification_Lock(int,int,int,int);
WINSHELLAPI int WINAPI SHChangeNotification_Unlock(int);
//SHChangeNotify(int,int,int,int);
WINSHELLAPI int WINAPI SHChangeNotifyDeregister(int);
WINSHELLAPI int WINAPI SHChangeNotifyReceive(int,int,int,int);
WINSHELLAPI int WINAPI SHChangeNotifyRegister(int,int,int,int,int,int);
WINSHELLAPI int WINAPI SHChangeNotifyUpdateEntryList(int,int,int,int);
WINSHELLAPI int WINAPI SHChangeRegistrationReceive(int,int);
WINSHELLAPI int WINAPI SHCloneSpecialIDList(int,int,int);
WINSHELLAPI int WINAPI SHCoCreateInstance(int,int,int,int,int);
WINSHELLAPI int WINAPI SHCreateDefClassObject(int,int,int,int,int);
WINSHELLAPI int WINAPI SHCreateDirectory(int,int);
WINSHELLAPI int WINAPI SHCreateLinks(int,int,int,int,int);
WINSHELLAPI int WINAPI SHCreatePropSheetExtArray(int,int,int);
//WINSHELLAPI int WINAPI SHCreateShellFolderViewEx(int,int);
WINSHELLAPI int WINAPI SHCreateStdEnumFmtEtc(int,int,int);
WINSHELLAPI int WINAPI SHDestroyPropSheetExtArray(int);
WINSHELLAPI int WINAPI SHDllGetClassObject(int,int,int);
WINSHELLAPI int WINAPI SHDoDragDrop(int,int,int,int,int);
//SHFileOperation(int);
//SHFileOperationA(int);
//SHFileOperationW(int);
WINSHELLAPI int WINAPI SHFindComputer(int,int);
WINSHELLAPI int WINAPI SHFindFiles(int,int);
WINSHELLAPI int WINAPI SHFind_InitMenuPopup(int,int,int,int);
WINSHELLAPI int WINAPI SHFlushClipboard(void);
WINSHELLAPI int WINAPI SHFormatDrive(int,int,int,int);
//WINSHELLAPI int WINAPI SHFree(int);
//SHFreeNameMappings(int);
WINSHELLAPI int WINAPI SHFreeShared(int,int);
WINSHELLAPI int WINAPI SHFreeUnusedLibraries(void);
//SHGetDataFromIDListA(int,int,int,int,int);
//SHGetDesktopFolder(int);
WINSHELLAPI int WINAPI SHGetFileIcon(int,int,int,int);
//SHGetFileInfo(int,int,int,int,int);
//SHGetFileInfoA(int,int,int,int,int);
//SHGetFileInfoW(int,int,int,int,int);
//SHGetInstanceExplorer(int);
//SHGetMalloc(int);
WINSHELLAPI int WINAPI SHGetNetResource(int,int,int,int);
WINSHELLAPI int WINAPI SHGetNewLinkInfo(int,int,int,int,int);
WINSHELLAPI int WINAPI SHGetNewLinkInfoA(int,int,int,int,int);
WINSHELLAPI int WINAPI SHGetNewLinkInfoW(int,int,int,int,int);
//SHGetPathFromIDList(int,int);
//SHGetPathFromIDListA(int,int);
//SHGetPathFromIDListW(int,int);
WINSHELLAPI int WINAPI SHGetRealIDL(int,int,int);
WINSHELLAPI int WINAPI SHGetSetSettings(int,int,int);
//SHGetSpecialFolderLocation(int,int,int);
WINSHELLAPI int WINAPI SHGetSpecialFolderPath(int,int,int,int);
WINSHELLAPI int WINAPI SHGlobalDefect(int);
WINSHELLAPI int WINAPI SHHandleDiskFull(int,int);
WINSHELLAPI int WINAPI SHHelpShortcuts_RunDLL(int,int,int,int);
WINSHELLAPI int WINAPI SHHelpShortcuts_RunDLLA(int,int,int,int);
WINSHELLAPI int WINAPI SHHelpShortcuts_RunDLLW(int,int,int,int);
WINSHELLAPI int WINAPI SHILCreateFromPath(int,int,int);
WINSHELLAPI int WINAPI SHIsBadInterfacePtr(int,int);
//SHLoadInProc(int);
WINSHELLAPI int WINAPI SHLoadOLE(int);
WINSHELLAPI int WINAPI SHLocalAlloc(int,int);
WINSHELLAPI int WINAPI SHLocalFree(int);
WINSHELLAPI int WINAPI SHLocalReAlloc(int,int,int);
WINSHELLAPI int WINAPI SHLockShared(int,int);
WINSHELLAPI int WINAPI SHLogILFromFSIL(int);
WINSHELLAPI int WINAPI SHMapPIDLToSystemImageListIndex(int,int,int);
WINSHELLAPI int WINAPI SHNetConnectionDialog(int,int,int);
WINSHELLAPI int WINAPI SHObjectProperties(int,int,int,int);
WINSHELLAPI int WINAPI SHOutOfMemoryMessageBox(int,int,int);
WINSHELLAPI int WINAPI SHRegCloseKey(int);
WINSHELLAPI int WINAPI SHRegDeleteKeyW(int,int);
WINSHELLAPI int WINAPI SHRegOpenKeyA(int,int,int);
WINSHELLAPI int WINAPI SHRegOpenKeyW(int,int,int);
WINSHELLAPI int WINAPI SHRegQueryValueA(int,int,int,int);
WINSHELLAPI int WINAPI SHRegQueryValueExA(int,int,int,int,int,int);
WINSHELLAPI int WINAPI SHRegQueryValueExW(int,int,int,int,int,int);
WINSHELLAPI int WINAPI SHRegQueryValueW(int,int,int,int);
WINSHELLAPI int WINAPI SHRegisterDragDrop(int,int);
WINSHELLAPI int WINAPI SHReplaceFromPropSheetExtArray(int,int,int,int);
WINSHELLAPI int WINAPI SHRestricted(int);
WINSHELLAPI int WINAPI SHRevokeDragDrop(int);
WINSHELLAPI int WINAPI SHRunControlPanel(int,int);
WINSHELLAPI int WINAPI SHSetInstanceExplorer(int);

// int SHShellFolderView_Message(HWND hwndCabinet,UINT uMsg, int nCol);
//
// hwndCabinet
//		This hwnd defines the explorer cabinet window that contains the 
//		shellview you need to communicate with.
// uMsg
//		A parameter identifying the SFVM enum to perform.
//	lParam
//		Message dependent parameter
//
//  Return: Unknown
//
//  Messages that can be sent:
// 
//  SFVM_REARRANGE
//    This message gets sent when a column gets clicked to instruct the
//    shell view to re-sort the item list. lParam identifies the column
//    that was clicked.

#define SFVM_REARRANGE	1

WINSHELLAPI int WINAPI SHShellFolderView_Message(HWND hwndCabinet,UINT uMsg, LPARAM lParam);

//

WINSHELLAPI int WINAPI SHSimpleIDListFromPath(int);
WINSHELLAPI int WINAPI SHStartNetConnectionDialog(int,int,int);
WINSHELLAPI int WINAPI SHUnlockShared(int);
WINSHELLAPI int WINAPI SHValidateUNC(int,int,int);
WINSHELLAPI int WINAPI SHWaitForFileToOpen(int,int,int);
WINSHELLAPI int WINAPI SHWaitOp_Operate(int,int);
WINSHELLAPI int WINAPI SHWinHelp(int,int,int,int);
WINSHELLAPI int WINAPI SetAppStartingCursor(int,int);
WINSHELLAPI int WINAPI SheChangeDirA(int);
WINSHELLAPI int WINAPI SheChangeDirExA(int);
WINSHELLAPI int WINAPI SheChangeDirExW(int);
WINSHELLAPI int WINAPI SheChangeDirW(int);
WINSHELLAPI int WINAPI SheConvertPathW(int,int,int);
WINSHELLAPI int WINAPI SheFullPathA(int,int,int);
WINSHELLAPI int WINAPI SheFullPathW(int,int,int);
WINSHELLAPI int WINAPI SheGetCurDrive(void);
WINSHELLAPI int WINAPI SheGetDirA(int,int);
WINSHELLAPI int WINAPI SheGetDirExW(int,int,int);
WINSHELLAPI int WINAPI SheGetDirW(int,int);
WINSHELLAPI int WINAPI SheGetPathOffsetW(int);
WINSHELLAPI int WINAPI SheRemoveQuotesA(int);
WINSHELLAPI int WINAPI SheRemoveQuotesW(int);
WINSHELLAPI int WINAPI SheSetCurDrive(int);
WINSHELLAPI int WINAPI SheShortenPathA(int,int);
WINSHELLAPI int WINAPI SheShortenPathW(int,int);
//ShellAboutA(int,int,int,int);
//ShellAboutW(int,int,int,int);
//ShellExecuteA(int,int,int,int,int,int);
//ShellExecuteEx(int);
//ShellExecuteExA(int);
//ShellExecuteExW(int);
//ShellExecuteW(int,int,int,int,int,int);
WINSHELLAPI int WINAPI ShellHookProc(int,int,int);
//ShellMessageBox
WINSHELLAPI int WINAPI Shell_GetCachedImageIndex(int,int,int);
WINSHELLAPI int WINAPI Shell_GetImageLists(int,int);
WINSHELLAPI int WINAPI Shell_MergeMenus(int,int,int,int,int,int);
//Shell_NotifyIcon(int,int);
//Shell_NotifyIconA(int,int);
//Shell_NotifyIconW(int,int);
WINSHELLAPI int WINAPI ShortSizeFormatW(int,int);
WINSHELLAPI int WINAPI SignalFileOpen(int);

// String functions as per SHWAPI.DLL
// WTF are they doing in shell32.lib ???
// (maybe they arn't anymore!)

WINSHELLAPI int WINAPI StrChrA(int,int);
WINSHELLAPI int WINAPI StrChrIA(int,int);
WINSHELLAPI int WINAPI StrChrIW(int,int);
WINSHELLAPI int WINAPI StrChrW(int,int);
WINSHELLAPI int WINAPI StrCmpNA(int,int,int);
WINSHELLAPI int WINAPI StrCmpNIA(int,int,int);
WINSHELLAPI int WINAPI StrCmpNIW(int,int,int);
WINSHELLAPI int WINAPI StrCmpNW(int,int,int);
WINSHELLAPI int WINAPI StrCpyNA(int,int,int);
WINSHELLAPI int WINAPI StrCpyNW(int,int,int);
WINSHELLAPI int WINAPI StrNCmpA(int,int,int);
WINSHELLAPI int WINAPI StrNCmpIA(int,int,int);
WINSHELLAPI int WINAPI StrNCmpIW(int,int,int);
WINSHELLAPI int WINAPI StrNCmpW(int,int,int);
WINSHELLAPI int WINAPI StrNCpyA(int,int,int);
WINSHELLAPI int WINAPI StrNCpyW(int,int,int);
WINSHELLAPI int WINAPI StrRChrA(int,int,int);
WINSHELLAPI int WINAPI StrRChrIA(int,int,int);
WINSHELLAPI int WINAPI StrRChrIW(int,int,int);
WINSHELLAPI int WINAPI StrRChrW(int,int,int);
WINSHELLAPI int WINAPI StrRStrA(int,int,int);
WINSHELLAPI int WINAPI StrRStrIA(int,int,int);
WINSHELLAPI int WINAPI StrRStrIW(int,int,int);
WINSHELLAPI int WINAPI StrRStrW(int,int,int);
WINSHELLAPI int WINAPI StrRetToStrN(int,int,int,int);
WINSHELLAPI int WINAPI StrStrA(int,int);
WINSHELLAPI int WINAPI StrStrIA(int,int);
WINSHELLAPI int WINAPI StrStrIW(int,int);
WINSHELLAPI int WINAPI StrStrW(int,int);
WINSHELLAPI int WINAPI StrToOleStr(int,int);
WINSHELLAPI int WINAPI StrToOleStrN(int,int,int,int);
WINSHELLAPI int WINAPI WOWShellExecute(int,int,int,int,int,int,int);
WINSHELLAPI int WINAPI Win32CreateDirectory(int,int);
WINSHELLAPI int WINAPI Win32DeleteFile(int);
WINSHELLAPI int WINAPI Win32RemoveDirectory(int);
WINSHELLAPI int WINAPI WriteCabinetState(int);

#ifdef __cplusplus
} //extern "C" {
#endif

#endif //_SHLEXT_H_
